# _Console::Virtual.pm - unbuffered-input/addressed-display layer
# v2003.0325 Chris Pressey, Cat's Eye Technologies

# Copyright (c)2003, Cat's Eye Technologies.  All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 
#   Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# 
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
# 
#   Neither the name of Cat's Eye Technologies nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission. 
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
# CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
# OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE. 

package _Console::Virtual;
BEGIN
{
  use 5;
  use strict qw(subs);
  use Exporter;
  $VERSION = 2003.0325;
  @ISA = qw(Exporter);
  @EXPORT_OK = qw(&display &clrscr &clreol &gotoxy
                  &bold &inverse &normal
                  &update_display &getkey &color);
}

%setup = ();

BEGIN
{
  my $c;
  my $fc = 0;  # found Curses.pm?
  my $fs = 0;  # found Term::Screen?
  my $fp = 0;  # found POSIX.pm?
  my $ft = 0;  # found $TERM and /etc/termcap?
  foreach $c (@INC)
  {
    $fc = 1 if -r "$c/Curses.pm";
    $fs = 1 if -r "$c/Term/Screen.pm";
    $fp = 1 if -r "$c/POSIX.pm";
  }
  $ft = $ENV{TERM} && -r "/etc/termcap";
  $| = 1;

  # Determine raw input module to use.
  # This can be pre-set by the calling code
  # by modifying %_Console::Virtual::setup.

  if (defined $setup{input})
  {
    require "_Console/Input/$setup{input}.pm";
  }
  elsif ($fc)
  {
    require _Console::Input::Curses;
  }
  elsif ($^O eq 'MSWin32')
  {
    require _Console::Input::Win32;
  }
  elsif ($fs)
  {
    require _Console::Input::Screen;
  }
  elsif ($fp)
  {
    require _Console::Input::POSIX;
  } else
  {
    warn "Warning! Raw input probably not available on this '$^O' system.\n";
    require _Console::Input::Teletype;
  }

  # Determine screen-addressed output method to use.
  # This can be pre-set by the calling code
  # by modifying %_Console::Virtual::setup.

  if (defined $setup{display})
  {
    require "_Console/Display/$setup{display}.pm";
  }
  elsif ($fc)
  {
    require _Console::Display::Curses;
  }
  elsif ($^O eq 'MSWin32')
  {
    require _Console::Display::Win32;
  }
  elsif ($fs)
  {
    require _Console::Display::Screen;
  }
  elsif ($ft)
  {
    require _Console::Display::Tput;
  } else
  {
    warn "Addressable screen must be emulated on this '$^O' system";
    require _Console::Display::Teletype;
  }

  # 2001.01.27 CAP
  # Determine color module to use.
  # This can be pre-set by the calling code
  # by modifying %_Console::Virtual::setup.

  if (defined $setup{color})
  {
    require "_Console/Color/$setup{color}.pm";
  }
  elsif ($fc)
  {
    require _Console::Color::Curses;
  }
  elsif ($^O eq 'MSWin32')
  {
    require _Console::Color::Win32;
  }
  elsif ($fs)
  {
    # require _Console::Color::Screen;  # TODO! needs to be written
    require _Console::Color::ANSI16;    # not a very general solution
  }
  else
  {
    require _Console::Color::Mono;
  }
}

1;

### END of Virtual.pm ###
